#include <windows.h>
#include "winapp.h"

WinApp *WinApp::TheWinApp;

int WinApp::Init(char* szNewAppName, HINSTANCE hNewInst, LPSTR lpszNewArgs, int nNewCmdShow )
{
	// Initialize data members
	szAppName = szNewAppName;
	hInst = hNewInst;
	lpszArgs = lpszNewArgs;
	nCmdShow = nNewCmdShow;

	// Define Window Class
	WndClass.hInstance = hInst;
	WndClass.lpszClassName = szAppName;
	WndClass.lpfnWndProc = WndProc;
	WndClass.style = 0;	// No special styles

	// Set resources for the program: Icon, Cursor, Menu (we have none), and background
	WndClass.hIcon = LoadIcon( NULL, IDI_APPLICATION );
	WndClass.hCursor = LoadCursor( NULL, IDC_ARROW );
	WndClass.lpszMenuName = NULL;
	WndClass.hbrBackground = (HBRUSH) GetStockObject( WHITE_BRUSH );

	WndClass.cbClsExtra = 0; // 'Extra' information about this window,
	WndClass.cbWndExtra = 0; // our program has none.

	// Register the class (this includes the 'WndProc')
	PreRegisterClass( WndClass );	// A hook, for derived classes
	if( !RegisterClass( &WndClass ) )
	{
		MessageBox( NULL, "Error registering class", NULL, MB_ICONEXCLAMATION|MB_OK );
		return( 0 );
	}

	// Create the window, which will be our main window
	hWnd = CreateWin( szAppName, "Hello", WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, 
		HWND_DESKTOP, NULL, hInst, NULL );

	if( !hWnd )
	{
		MessageBox( NULL, "Error creating window", NULL, MB_ICONEXCLAMATION|MB_OK );
		return( 0 );
	}
	return(1);
}

int WinApp::MessageLoop()
{
	while( GetMessage( &Msg, NULL, 0, 0 ) )
	{
		TranslateMessage( &Msg );
		DispatchMessage( &Msg );
	}
	return( Msg.wParam );
}

HWND WinApp::CreateWin( LPCTSTR lpClassName, LPCTSTR lpWindowName, DWORD dwStyle, 
	int x, int y, int nWidth, int nHeight, HWND hWndParent, 
	HMENU hMenu, HANDLE hInstance, LPVOID lpParam )
{
	return( CreateWindow ( lpClassName, lpWindowName, dwStyle, 
		x, y, nWidth, nHeight, hWndParent, hMenu, hInstance, lpParam ) );
}

int WinApp::Run(char* szAppName, HINSTANCE hNewInst, LPSTR lpszNewArgs, int nNewCmdShow )
{
	if( Init(szAppName, hNewInst, lpszNewArgs, nNewCmdShow ) == 0 )
		return(0);
	Show();
	return( MessageLoop() );
}

LRESULT CALLBACK WinApp::WndProc( HWND hWnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch( Message )
	{
		case WM_PAINT:
			HDC DC;
			PAINTSTRUCT PaintStruct;
			DC = BeginPaint( hWnd, &PaintStruct );
			TheWinApp->OnDraw( hWnd, DC, PaintStruct );
			EndPaint( hWnd, &PaintStruct );
			break;
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		default:
			int Return, Handled=0;
			Return = TheWinApp->OnMessage(hWnd, Message, wParam, lParam, Handled );
			if( Handled )
				return( Return );
			
			// Any unhandled messages are passed onto to the default handler.
			return( DefWindowProc( hWnd, Message, wParam, lParam ) );
	}
	return( 0 );
}
		
void WinApp::OnDraw( HWND& hWnd, HDC& DC, PAINTSTRUCT& PaintStruct ) 
{ 
	TextOut( DC, 0, 0, "WinApp::OnDraw", 14 );
}

